@@ -78,6 +78,8 @@ type verify_error =
   | Error_v_keyusage_no_certsign
   | Error_v_application_verification
 
+type bigarray = (char, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
+
 exception Method_error
 exception Context_error
 exception Certificate_error
@@ -139,8 +141,12 @@ external set_password_callback : context -> (bool -> string) -> unit = "ocaml_ss
 
 external embed_socket : Unix.file_descr -> context -> socket = "ocaml_ssl_embed_socket"
 
+external disable_protocols : context -> protocol list -> unit = "ocaml_ssl_disable_protocols"
+
 external set_cipher_list : context -> string -> unit = "ocaml_ssl_ctx_set_cipher_list"
 
+external honor_cipher_order : context -> unit = "ocaml_ssl_ctx_honor_cipher_order"
+
 external init_dh_from_file : context -> string -> unit = "ocaml_ssl_ctx_init_dh_from_file"
 
 external init_ec_from_named_curve : context -> string -> unit = "ocaml_ssl_ctx_init_ec_from_named_curve"
@@ -160,6 +166,8 @@ external get_client_verify_callback_ptr : unit -> verify_callback = "ocaml_ssl_g
 
 let client_verify_callback = get_client_verify_callback_ptr ()
 
+external set_client_verify_callback_verbose : bool -> unit = "ocaml_ssl_set_client_verify_callback_verbose"
+
 external set_verify : context -> verify_mode list -> verify_callback option -> unit = "ocaml_ssl_ctx_set_verify"
 
 external set_verify_depth : context -> int -> unit = "ocaml_ssl_ctx_set_verify_depth"
@@ -196,9 +204,20 @@ external connect : socket -> unit = "ocaml_ssl_connect"
 
 external verify : socket -> unit = "ocaml_ssl_verify"
 
-external write : socket -> string -> int -> int -> int = "ocaml_ssl_write"
+external write : socket -> Bytes.t -> int -> int -> int = "ocaml_ssl_write"
 
-external read : socket -> string -> int -> int -> int = "ocaml_ssl_read"
+external write_bigarray : socket -> bigarray -> int -> int -> int = "ocaml_ssl_write_bigarray"
+
+external write_bigarray_blocking :
+  socket -> bigarray -> int -> int -> int = "ocaml_ssl_write_bigarray_blocking"
+
+external read : socket -> Bytes.t -> int -> int -> int = "ocaml_ssl_read"
+
+external read_into_bigarray :
+  socket -> bigarray -> int -> int -> int = "ocaml_ssl_read_into_bigarray"
+
+external read_into_bigarray_blocking :
+  socket -> bigarray -> int -> int -> int = "ocaml_ssl_read_into_bigarray_blocking"
 
 external accept : socket -> unit = "ocaml_ssl_accept"
 
@@ -230,43 +249,43 @@ let output_string ssl s =
   ignore (write ssl s 0 (String.length s))
 
 let output_char ssl c =
-  let tmp = String.create 1 in
-    tmp.[0] <- c;
+  let tmp = Bytes.create 1 in
+    Bytes.set tmp 0 c;
     ignore (write ssl tmp 0 1)
 
 let output_int ssl i =
-  let tmp = String.create 4 in
-    tmp.[0] <- char_of_int (i lsr 24);
-    tmp.[1] <- char_of_int ((i lsr 16) land 0xff);
-    tmp.[2] <- char_of_int ((i lsr 8) land 0xff);
-    tmp.[3] <- char_of_int (i land 0xff);
+  let tmp = Bytes.create 4 in
+    Bytes.set tmp 0 (char_of_int (i lsr 24));
+    Bytes.set tmp 1 (char_of_int ((i lsr 16) land 0xff));
+    Bytes.set tmp 2 (char_of_int ((i lsr 8) land 0xff));
+    Bytes.set tmp 3 (char_of_int (i land 0xff));
     if write ssl tmp 0 4 <> 4 then failwith "output_int error: all the byte were not sent"
 
 let input_string ssl =
   let bufsize = 1024 in
-  let buf = String.create bufsize in
+  let buf = Bytes.create bufsize in
   let ret = ref "" in
   let r = ref 1 in
     while !r <> 0
     do
       r := read ssl buf 0 bufsize;
-      ret := !ret ^ (String.sub buf 0 !r)
+      ret := !ret ^ (Bytes.sub buf 0 !r)
     done;
     !ret
 
 let input_char ssl =
-  let tmp = String.create 1 in
+  let tmp = Bytes.create 1 in
     if read ssl tmp 0 1 <> 1 then
       raise End_of_file
     else
-      tmp.[0]
+      Bytes.get tmp 0
 
 let input_int ssl =
   let i = ref 0 in
-  let tmp = String.create 4 in
+  let tmp = Bytes.create 4 in
     ignore (read ssl tmp 0 4);
-    i := int_of_char (tmp.[0]);
-    i := (!i lsl 8) + int_of_char (tmp.[1]);
-    i := (!i lsl 8) + int_of_char (tmp.[2]);
-    i := (!i lsl 8) + int_of_char (tmp.[3]);
+    i := int_of_char (Bytes.get tmp 0);
+    i := (!i lsl 8) + int_of_char (Bytes.get tmp 1);
+    i := (!i lsl 8) + int_of_char (Bytes.get tmp 2);
+    i := (!i lsl 8) + int_of_char (Bytes.get tmp 3);
     !i
